package com.fjhx.victoriatourist.service.logistics.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.area.utils.CustomizeAreaUtil;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.common.utils.Assert;
import com.fjhx.item.util.excel.util.ExcelUtil;
import com.fjhx.kd100.entity.company.po.CompanyInfo;
import com.fjhx.kd100.entity.config.po.ConfigInfo;
import com.fjhx.kd100.entity.logistics.po.LogisticsInfo;
import com.fjhx.kd100.event.Kd100CallbackEvent;
import com.fjhx.kd100.service.company.CompanyInfoService;
import com.fjhx.kd100.service.config.ConfigInfoService;
import com.fjhx.kd100.util.KD100Result;
import com.fjhx.kd100.util.KD100Util;
import com.fjhx.purchase.entity.purchase.po.Purchase;
import com.fjhx.purchase.service.purchase.PurchaseService;
import com.fjhx.victoriatourist.entity.deliver.po.DeliverGoods;
import com.fjhx.victoriatourist.entity.logistics.LogisticsInfosExcelBo;
import com.fjhx.victoriatourist.entity.logistics.LogisticsProductInfo;
import com.fjhx.victoriatourist.entity.logistics.dto.LogisticsInfosDto;
import com.fjhx.victoriatourist.entity.logistics.dto.LogisticsInfosSelectDto;
import com.fjhx.victoriatourist.entity.logistics.po.LogisticsDetails;
import com.fjhx.victoriatourist.entity.logistics.po.LogisticsInfos;
import com.fjhx.victoriatourist.entity.logistics.vo.LogisticsInfosVo;
import com.fjhx.victoriatourist.mapper.logistics.LogisticsInfosMapper;
import com.fjhx.victoriatourist.service.deliver.DeliverGoodsService;
import com.fjhx.victoriatourist.service.jd.JdBackService;
import com.fjhx.victoriatourist.service.logistics.LogisticsDetailsService;
import com.fjhx.victoriatourist.service.logistics.LogisticsInfosService;
import com.fjhx.victoriatourist.utils.LogisticsConstant;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.framework.mybatis.holder.TenantHolder;
import com.ruoyi.system.service.ISysRoleService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 物流信息 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-17
 */
@Slf4j
@Service
public class LogisticsInfosServiceImpl extends ServiceImpl<LogisticsInfosMapper, LogisticsInfos> implements LogisticsInfosService {
    @Autowired
    private CompanyInfoService companyInfoService;
    @Autowired
    private LogisticsDetailsService logisticsDetailsService;
    @Autowired
    private PurchaseService purchaseService;
    @Autowired
    private JdBackService jdBackService;
    @Autowired
    private ISysRoleService sysRoleService;
    @Autowired
    private ConfigInfoService configInfoService;
    @Autowired
    private DeliverGoodsService deliverGoodsService;

    private void setPageWrapper(IWrapper<LogisticsInfos> wrapper, LogisticsInfosSelectDto dto) {
        wrapper.eq("li", LogisticsInfos::getId, dto.getId());

        wrapper.eq("li", LogisticsInfos::getBusinessType, dto.getBusinessType());
        wrapper.eq("li", LogisticsInfos::getLogisticsStatus, dto.getLogisticsStatus());
        wrapper.eq("li", LogisticsInfos::getStatus, dto.getStatus());
        if (ObjectUtil.isNotEmpty(dto.getKeyword())) {
            //判断是否是 采购到货 采购退货 来源
            boolean flag = !Objects.equals(dto.getBusinessType(), 1) && !Objects.equals(dto.getBusinessType(), 5);

            List<String> companyCodes = companyInfoService.listObject(CompanyInfo::getCode, q -> q.like(CompanyInfo::getName, dto.getKeyword()));
            wrapper.and(q -> q.like("li", LogisticsInfos::getCode, dto.getKeyword())
                    .or().like("li", LogisticsInfos::getSubCode, dto.getKeyword())
                    .or().like("IF( li.business_type IN ( 1, 5 ), p.contractCode, li.business_code ) ", dto.getKeyword())
                    .or().in("li", LogisticsInfos::getLogisticsCompanyCode, companyCodes)
                    .or(flag).like(flag, "pi.custom_code", dto.getKeyword())
                    .or(flag).like(flag, "pi.name", dto.getKeyword())
                    //采购到货 采购退货 来源
                    .or().inSql("p.id", "select pd.purchase_id from bytesailing_purchase.purchase_detail pd LEFT JOIN bytesailing_item.product_info pi ON pd.bussiness_id = pi.id where ( pi.`name` LIKE CONCAT( '%', '" + dto.getKeyword() + "', '%' ) OR pi.custom_code LIKE CONCAT( '%', '" + dto.getKeyword() + "', '%' ))")
            );
        }

        //添加权限过滤
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        Set<String> permissionList = sysRoleService.selectRolePermissionByUserId(SecurityUtils.getUserId());
        DynamicDataSourceContextHolder.poll();
        if (ObjectUtil.isNotEmpty(permissionList)) {
            //if当前用户的角色 in [运营专员, 运营助理] 页面仅展示产品.管理部门 == 当前用户所在部门或下级部门的相关数据
            if (permissionList.contains("E-commerce operation") || permissionList.contains("Operation assistant")) {
                wrapper.and(q -> q.eq("de.dept_id", SecurityUtils.getDeptId())
                        .or()
                        .like("de.ancestors", SecurityUtils.getDeptId()));
            }
        }

        //部门(项目组)过滤
        wrapper.eq("de.dept_id", dto.getDeptId());

        //时间范围过滤
        wrapper.ge("li.create_time", dto.getBeginTime());
        wrapper.le("li.create_time", dto.getEndTime());


        wrapper.groupBy("li.id");

        wrapper.orderByDesc("li", LogisticsInfos::getId);
        Assert.notEmpty(dto.getBusinessType(), "物流数据来源类型不能为空");
    }

    @Override
    public Page<LogisticsInfosVo> getPage(LogisticsInfosSelectDto dto) {
        IWrapper<LogisticsInfos> wrapper = getWrapper();

        //设置通用查询条件
        setPageWrapper(wrapper, dto);

        Page<LogisticsInfosVo> page = this.baseMapper.getPage(dto.getPage(), wrapper, dto.getBusinessType());
        List<LogisticsInfosVo> records = page.getRecords();

        //赋值通用信息
        setPageInfo(records);

        return page;
    }

    private void setPageInfo(List<LogisticsInfosVo> records) {
        if (ObjectUtil.isEmpty(records)) {
            return;
        }
        //赋值物流公司名字
        List<String> codes = records.stream().map(LogisticsInfosVo::getLogisticsCompanyCode).collect(Collectors.toList());
        if (ObjectUtil.isEmpty(codes)) {
            return;
        }
        List<CompanyInfo> list = companyInfoService.list(q -> q.in(CompanyInfo::getCode, codes));
        if (ObjectUtil.isNotEmpty(list)) {
            Map<String, String> companyMap = list.stream().collect(Collectors.toMap(CompanyInfo::getCode, CompanyInfo::getName));
            for (LogisticsInfosVo record : records) {
                String name = companyMap.get(record.getLogisticsCompanyCode());
                record.setLogisticsCompanyName(name);
            }
        }
        //赋值国省市名称
        CustomizeAreaUtil.setAreaName(records);

        //赋值产品信息
        setProductInfo(records);
    }

    /**
     * 赋值物流关联业务产品信息
     */
    private void setProductInfo(List<LogisticsInfosVo> records) {
        if (ObjectUtil.isEmpty(records)) {
            return;
        }
        //采购到货
        {
            List<LogisticsInfosVo> collect0 = records.stream().filter(item -> item.getBusinessType().equals(1)).collect(Collectors.toList());
            List<Long> ids0 = collect0.stream().map(LogisticsInfos::getId).distinct().collect(Collectors.toList());
            if (ObjectUtil.isNotEmpty(ids0)) {
                List<LogisticsProductInfo> byDeliverGoods = baseMapper.getLogisticsProductInfoByDeliverGoods(IWrapper.getWrapper()
                        .in("lis.id", ids0)
                );
                Map<Long, List<LogisticsProductInfo>> byDeliverGoodsMap = byDeliverGoods.stream().collect(Collectors.groupingBy(LogisticsProductInfo::getId));
                for (LogisticsInfosVo logisticsInfosVo : collect0) {
                    logisticsInfosVo.setLogisticsProductInfoList(byDeliverGoodsMap.getOrDefault(logisticsInfosVo.getId(), new ArrayList<>()));
                }
            }
        }

        //京东数据
        {
            List<LogisticsInfosVo> collect1 = records.stream().filter(item -> item.getBusinessType().equals(2)).collect(Collectors.toList());
            List<Long> ids1 = collect1.stream().map(LogisticsInfos::getId).distinct().collect(Collectors.toList());
            if (ObjectUtil.isNotEmpty(ids1)) {
                List<LogisticsProductInfo> byStockWait = baseMapper.getLogisticsProductInfoByStockWait(IWrapper.getWrapper()
                        .in("lis.id", ids1)
                );
                Map<Long, List<LogisticsProductInfo>> byStockWaitMap = byStockWait.stream().collect(Collectors.groupingBy(LogisticsProductInfo::getId));
                for (LogisticsInfosVo logisticsInfosVo : collect1) {
                    logisticsInfosVo.setLogisticsProductInfoList(byStockWaitMap.getOrDefault(logisticsInfosVo.getId(), new ArrayList<>()));
                }
            }
        }

        //采购退货
        {
            List<LogisticsInfosVo> collect2 = records.stream().filter(item -> item.getBusinessType().equals(5)).collect(Collectors.toList());
            List<Long> ids2 = collect2.stream().map(LogisticsInfos::getId).distinct().collect(Collectors.toList());
            if (ObjectUtil.isNotEmpty(ids2)) {
                List<LogisticsProductInfo> byStockWait1 = baseMapper.getLogisticsProductInfoByStockWait(IWrapper.getWrapper()
                        .in("lis.id", ids2)
                );
                Map<Long, List<LogisticsProductInfo>> byStockWaitMap1 = byStockWait1.stream().collect(Collectors.groupingBy(LogisticsProductInfo::getId));
                for (LogisticsInfosVo logisticsInfosVo : collect2) {
                    logisticsInfosVo.setLogisticsProductInfoList(byStockWaitMap1.getOrDefault(logisticsInfosVo.getId(), new ArrayList<>()));
                }
            }
        }
    }

    @Override
    public LogisticsInfosVo detail(Long id) {
        LogisticsInfos LogisticsInfos = this.getById(id);
        LogisticsInfosVo result = BeanUtil.toBean(LogisticsInfos, LogisticsInfosVo.class);
        //赋值国省市名称
        CustomizeAreaUtil.setAreaName(Collections.singletonList(result));
        return result;
    }

    @DSTransactional
    @Override
    public void add(LogisticsInfosDto logisticsInfosDto) {
        long logisticsInfosId = IdWorker.getId();
        logisticsInfosDto.setId(logisticsInfosId);
        if (logisticsInfosDto.getIsKd100() == 1) {
            //发起快递100订阅
            DynamicDataSourceContextHolder.push(SourceConstant.KD100);
            Integer state = KD100Util.monitor(logisticsInfosDto.getLogisticsCompanyCode(), logisticsInfosDto.getCode(), logisticsInfosId, 1, LogisticsInfos.class).getState();
            logisticsInfosDto.setLogisticsStatus(state);
            logisticsInfosDto.setIsKd100(1);
            DynamicDataSourceContextHolder.poll();
        }
        this.save(logisticsInfosDto);
        LogisticsDetails logisticsDetails = new LogisticsDetails();
        logisticsDetails.setLogisticsInfoId(logisticsInfosDto.getId());
        logisticsDetails.setRemark(logisticsInfosDto.getRemark());
        logisticsDetailsService.save(logisticsDetails);
    }

    @DSTransactional
    @EventListener
    public void kd100EventListener(Kd100CallbackEvent event) {
        LogisticsInfo logisticsInfo = event.getLogisticsInfo();
        //监听物流状态变化
        if (KD100Util.getTableName(LogisticsInfos.class).equals(logisticsInfo.getTableName())) {
            DynamicDataSourceContextHolder.push(SourceConstant.VICTORIATOURIST);
            LogisticsInfos byId = this.getById(logisticsInfo.getBusinessId());
            DynamicDataSourceContextHolder.poll();
            if (ObjectUtil.notEqual(logisticsInfo.getState(), byId.getLogisticsStatus())) {
                byId.setLogisticsStatus(logisticsInfo.getState());
                DynamicDataSourceContextHolder.push(SourceConstant.VICTORIATOURIST);
                this.updateById(byId);
                DynamicDataSourceContextHolder.poll();
                //如果是京东退货且物流状态为完成
                jdBackService.updateJdBack(byId);
            }
        }
    }

    @Override
    public void edit(LogisticsInfosDto logisticsInfosDto) {
        //只在生产环境调用Kd100
        String activeProfile = SpringUtil.getActiveProfile();

        if ("prod".equals(activeProfile) && logisticsInfosDto.getIsKd100() == 1) {
            //发起快递100订阅
            DynamicDataSourceContextHolder.push(SourceConstant.KD100);
            Integer state = KD100Util.monitor(logisticsInfosDto.getLogisticsCompanyCode(), logisticsInfosDto.getCode(), logisticsInfosDto.getId(), logisticsInfosDto.getBusinessType(), LogisticsInfos.class).getState();
            logisticsInfosDto.setLogisticsStatus(state);
            logisticsInfosDto.setIsKd100(1);
            DynamicDataSourceContextHolder.poll();
        }

        this.updateById(logisticsInfosDto);

        //更新发货单号
        if (ObjectUtil.equals(logisticsInfosDto.getBusinessType(), 1)) {
            deliverGoodsService.update(q -> q
                    .eq(DeliverGoods::getId, logisticsInfosDto.getBusinessId())
                    .or().eq(DeliverGoods::getId, logisticsInfosDto.getDeliverGoodsId())
                    .set(DeliverGoods::getLogisticsCompanyCode, logisticsInfosDto.getLogisticsCompanyCode())
                    .set(DeliverGoods::getCode, logisticsInfosDto.getCode())
            );
        }
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public KD100Result getLogistics(Long id) {
        LogisticsInfos logisticsInfo = this.getById(id);
        Assert.notEmpty(logisticsInfo, "没有找到物流信息");
        return KD100Util.queryTrack(logisticsInfo.getLogisticsCompanyCode(), logisticsInfo.getCode(), logisticsInfo.getPhone());
    }

    @Override
    @DSTransactional
    public void dataInfoEdit(Long id) {
        LogisticsInfos logisticsInfo = getById(id);
        Assert.notEmpty(logisticsInfo, "没有找到物流信息");
        KD100Result kd100Result = KD100Util.queryTrack(logisticsInfo.getLogisticsCompanyCode(), logisticsInfo.getCode(), logisticsInfo.getPhone());
        Integer state = kd100Result.getState();
        if (ObjectUtil.notEqual(state, logisticsInfo.getLogisticsStatus())) {
            //原来是异常物流信息 限制刷新到状态就添加订阅
            if (logisticsInfo.getLogisticsStatus() == -1) {
                KD100Util.subscribe(logisticsInfo.getLogisticsCompanyCode(), logisticsInfo.getCode());
            }
            logisticsInfo.setLogisticsStatus(state);
            updateById(logisticsInfo);
            //如果是京东退货且物流状态为完成
            jdBackService.updateJdBack(logisticsInfo);
        }
    }

    @Override
    public void arrivalNotice(LogisticsInfosVo entity) {
        // 采购id
        Long purchaseId = entity.getPurchaseId();
        Assert.notEmpty(purchaseId, "采购id不能为空");

        entity.setStatus(LogisticsConstant.Status.STATUS_1);
        updateById(entity);

        Purchase purchase = new Purchase();
        purchase.setId(purchaseId);
        purchase.setArrivalStatus(10);//设置部分到货
        purchaseService.updateById(purchase);

        //物流到货通知
        Assert.notEmpty(entity.getId(), "物流id不能为空");
        LogisticsInfos logisticsInfo = new LogisticsInfos();
        logisticsInfo.setId(entity.getId());
        logisticsInfo.setLogisticsStatus(15);
        updateById(logisticsInfo);
    }

    /**
     * 获取物流关联产品部门列表
     */
    @Override
    public List<SysDept> getDepts() {
        return baseMapper.getDepts();
    }

    /**
     * 每天0点刷新近3天的异常物流信息
     * "0 0/1 * * * ?"
     */
    @Scheduled(cron = "0 0 0 * * ?")
    void updateLast3DayInfo() {
        String tenantId = "wdly";
        TenantHolder.setIgnore(true);
        ConfigInfo configInfo = configInfoService.getOne(q -> q.eq(ConfigInfo::getTenantId, tenantId));
        if (configInfo == null) {
            return;
        }
        DynamicDataSourceContextHolder.push(SourceConstant.VICTORIATOURIST);
        IWrapper<LogisticsInfos> wrapper = IWrapper.getWrapper();
        wrapper.eq(LogisticsInfos::getLogisticsStatus, -1);
        wrapper.eq(LogisticsInfos::getIsKd100, 1);
        wrapper.eq("tenant_id", tenantId);
        wrapper.apply("DATE_SUB( CURDATE(), INTERVAL 3 DAY )<= create_time");
        List<LogisticsInfos> list = this.list(wrapper);
        if (ObjectUtil.isEmpty(list)) {
            return;
        }
        for (LogisticsInfos logisticsInfos : list) {
            String com = logisticsInfos.getLogisticsCompanyCode();
            String num = logisticsInfos.getCode();
            try {
                KD100Result kd100Result = KD100Util.queryTrack(com, num, logisticsInfos.getPhone(), configInfo);
                if (kd100Result.getState() == null || kd100Result.getState() == -1) {
                    continue;
                }
                logisticsInfos.setLogisticsStatus(kd100Result.getState());
                //重新订阅
                KD100Util.subscribe(com, num, configInfo);
                Thread.sleep(2000);
            } catch (Exception e) {
                log.error("重新同步异常物流信息出错： com:{}, num:{}, message:{}", com, num, e.getMessage(), e);
            }
        }
        this.updateBatchById(list);
        DynamicDataSourceContextHolder.clear();
        TenantHolder.clear();
    }

    @Override
    public void exportExcel(LogisticsInfosSelectDto dto, HttpServletResponse httpServletResponse) {
        IWrapper<LogisticsInfos> wrapper = getWrapper();
        setPageWrapper(wrapper, dto);
        List<LogisticsInfosVo> list = baseMapper.getList(wrapper, dto.getBusinessType());
        setPageInfo(list);

        //导出Excel
        List<LogisticsInfosExcelBo> logisticsInfosExcelBoList = BeanUtil.copyToList(list, LogisticsInfosExcelBo.class);
        for (LogisticsInfosExcelBo logisticsInfosExcelBo : logisticsInfosExcelBoList) {
            //数据来源
            if (logisticsInfosExcelBo.getBusinessType().equals("1")) {
                logisticsInfosExcelBo.setBusinessType("采购入库");
            }
            if (logisticsInfosExcelBo.getBusinessType().equals("2")) {
                logisticsInfosExcelBo.setBusinessType("京东订单出库");
            }
            if (logisticsInfosExcelBo.getBusinessType().equals("5")) {
                logisticsInfosExcelBo.setBusinessType("采购退货");
            }
            // 物流状态 -1未查询到快递信息 0在途，1揽收，2疑难，3签收，4退签，5派件，6退回，7转投，8清关，14拒签，15完成
            String logisticsStatus = logisticsInfosExcelBo.getLogisticsStatus();
            if (ObjectUtil.isNotEmpty(logisticsStatus)) {
                switch (logisticsStatus) {
                    case "-1":
                        logisticsStatus = "未知";
                        break;
                    case "0":
                        logisticsStatus = "在途";
                        break;
                    case "1":
                        logisticsStatus = "揽收";
                        break;
                    case "2":
                        logisticsStatus = "疑难";
                        break;
                    case "3":
                        logisticsStatus = "签收";
                        break;
                    case "4":
                        logisticsStatus = "退签";
                        break;
                    case "5":
                        logisticsStatus = "派件";
                        break;
                    case "6":
                        logisticsStatus = "退回";
                        break;
                    case "7":
                        logisticsStatus = "转投";
                        break;
                    case "8":
                        logisticsStatus = "清关";
                        break;
                    case "14":
                        logisticsStatus = "拒签";
                        break;
                    case "15":
                        logisticsStatus = "完成";
                        break;
                }
                logisticsInfosExcelBo.setLogisticsStatus(logisticsStatus);
            }

            //物流产品信息
            List<LogisticsProductInfo> logisticsProductInfoList = logisticsInfosExcelBo.getLogisticsProductInfoList();
            StringJoiner sj1 = new StringJoiner("\r\n");
            StringJoiner sj2 = new StringJoiner("\r\n");
            StringJoiner sj3 = new StringJoiner("\r\n");

            for (LogisticsProductInfo logisticsProductInfo : logisticsProductInfoList) {
                sj1.add(logisticsProductInfo.getProductCode());
                sj2.add(logisticsProductInfo.getProductName());
                sj3.add(ObjectUtil.defaultIfNull(logisticsProductInfo.getQuantity(), BigDecimal.ZERO).toString());
            }
            logisticsInfosExcelBo.setSku(sj1.toString());
            logisticsInfosExcelBo.setProductName(sj2.toString());
            logisticsInfosExcelBo.setQuantity(sj3.toString());
        }


        ExcelUtil.export(httpServletResponse, logisticsInfosExcelBoList, LogisticsInfosExcelBo.class);
    }

}
